
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Locale;

import javax.swing.JFrame;
import javax.swing.JPanel;




/**
 * <p>JFrame-Beispiel, Darstellung der komplexen
 *    Zahlenebene, MouseClick- und MouseMotion-Events werden ber innere 
 *    und anonyme Adapterklassen behandelt. 
 *    Zustzlich Operatoren und Darstellung des komplexen Winkels.
 * </p>
 * <p>Copyright: (c) 2003, 2018</p>
 * <p>Organisation: THK </p>
 * @author Wolfgang Konen
 * @version 2.0
 */
 
public  class GUIComplexPlane4 extends JFrame
{
    private static enum OPERAND {I_KONJ, I_HOCH2, I_HOCH3, I_HOCH1_2, I_HOCH1_3, I_HOCH1_4};
    private static final String[] opText = {"konjugieren","Quadrat","hoch 3","hoch 1/2","hoch 1/3","hoch 1/4"};
	private static final long serialVersionUID = 1L;
	private static final int WINWIDTH=465, WINHEIGHT=465, LABHEIGHT=25;
	private OPERAND opState;
	private double re=0.0;
	private double im=0.0;
	private TextField tfRe;
	private TextField tfIm;
	private Choice cbox;
	private MyWindow win;

	/**
	 * @param args
	 * @throws IOException 
	 */
	public static void main(String[] args) throws IOException 
	{
		GUIComplexPlane4 g_Frame = new GUIComplexPlane4("GUI Complex Plane");

		if (args.length!=0) {
			throw new RuntimeException("[GUIComplexPlane4.main] args="+args+" not allowed.");
		}

	}

	public GUIComplexPlane4() {
		init();
	}

	public GUIComplexPlane4(String title) {
		super(title);
		init();
	}
	
	public void init() {
		Font font = new Font("Arial",Font.PLAIN,16);
		addWindowListener(new WindowClosingAdapter());
		setSize(WINWIDTH,WINHEIGHT);		
		setBounds(100,100,WINWIDTH,WINHEIGHT);

//        enableEvents(AWTEvent.MOUSE_EVENT_MASK | AWTEvent.MOUSE_MOTION_EVENT_MASK);
		setBackground(Color.white);
		
		// Setzen der Initialparameter
		String strRe = "1.0";
		String strIm = "1.0";
		this.re = Double.valueOf(strRe).doubleValue();
		this.im = Double.valueOf(strIm).doubleValue();
		this.setLayout(null);
		tfRe = new TextField(strRe);
		tfIm = new TextField(strIm);
		tfRe.setBounds(107,WINHEIGHT-100,60,LABHEIGHT);
		tfIm.setBounds(107,WINHEIGHT-75,60,LABHEIGHT);
		tfRe.setFont(font);
		tfIm.setFont(font);
		this.add(tfRe);
		this.add(tfIm);
		Label lRe = new Label("Real:");
		Label lIm = new Label("Imag:");
		lRe.setBounds( 53,WINHEIGHT-100,40,LABHEIGHT);
		lIm.setBounds( 53,WINHEIGHT-75,40,LABHEIGHT);
		lRe.setBackground(new Color(250,250,230));
		lIm.setBackground(new Color(250,250,230));
		lRe.setFont(font);
		lIm.setFont(font);
		this.add(lRe);
		this.add(lIm);
		Label lChoice = new Label("Operator:");
		lChoice.setBounds(40,20,80,LABHEIGHT);
		lChoice.setFont(font);
		this.add(lChoice);
		cbox = new Choice();
		cbox.setBounds(120,20,130,LABHEIGHT);
		for (int i=0; i<opText.length; i++)
			cbox.addItem(opText[i]);
		cbox.addItemListener(new MyItemListener());
		cbox.setFont(font);
		this.add(cbox);
		cbox.select(opText[0]);
		opState = OPERAND.I_KONJ;
		setVisible(true);
		
		this.addMouseMotionListener(new MouseMotionAdapter() {
			public void mouseDragged(MouseEvent evt) {
				re = win.sx(evt.getX());
				im = win.sy(evt.getY());
				repaint();
			}
		});
		this.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent evt) {
				re = win.sx(evt.getX());
				im = win.sy(evt.getY());
				repaint();
			}
		});
		tfRe.addActionListener(new MyActionListener());
		tfIm.addActionListener(new MyActionListener());
	}
	
	class MyActionListener implements ActionListener {
		public void actionPerformed(ActionEvent evt) {
			re = Double.valueOf(tfRe.getText()).doubleValue();
			im = Double.valueOf(tfIm.getText()).doubleValue();
			repaint();
		}
		
	}
	class MyItemListener implements ItemListener {
		public void itemStateChanged(ItemEvent ie) {
			if (ie.getItem().equals(opText[0])) {opState = OPERAND.I_KONJ;}
			if (ie.getItem().equals(opText[1])) {opState = OPERAND.I_HOCH2;}
			if (ie.getItem().equals(opText[2])) {opState = OPERAND.I_HOCH3;}
			if (ie.getItem().equals(opText[3])) {opState = OPERAND.I_HOCH1_2;}
			if (ie.getItem().equals(opText[4])) {opState = OPERAND.I_HOCH1_3;}
			if (ie.getItem().equals(opText[5])) {opState = OPERAND.I_HOCH1_4;}
			repaint();
		}
	}
	
	public void paint(Graphics g)
	{
		int clip = 1;
		int width = getSize().width;
		int height = getSize().height;
		double xmin=-3.0, xmax=3.0;
		double ymin=-3.0, ymax=3.0;
		g.clearRect(clip,clip, width-2*clip, height-2*clip);
		g.setColor(Color.black);
		g.drawRect(clip,clip, width-2*clip, height-2*clip );
		g.setClip(clip,clip,  width-2*clip, height-2* clip);
		this.setBackground(new Color(250,250,230));  // helles gelb
		this.win = new MyWindow(g,xmin,ymin,xmax,ymax);
		
		// Koordinatengitter malen
		for (double i=xmin; i<(xmax+1);i+=0.25) {
			g.setColor(new Color(200,200,250));
			if (i%1.0==0.0) g.setColor(new Color(40,170,40));
			if (i==0.0) g.setColor(Color.blue);
			win.line(i,ymin,i,ymax);
		}
		for (double i=xmin; i<(xmax+1);i+=0.25) {
			g.setColor(new Color(200,200,250));
			if (i%1.0==0.0) g.setColor(new Color(40,170,40));
			if (i==0.0) g.setColor(Color.blue);
			win.line(xmin,i,xmax,i);
		}
		
		// komplexe Zahl (rot) zeigen
//		DecimalFormat form = new DecimalFormat("+0.000;-0.000");	// hat Komma als Dezimaltrennzeichen
		DecimalFormat df = new DecimalFormat();				
		df = (DecimalFormat) NumberFormat.getNumberInstance(Locale.UK);	// hat Punkt als Dez.-trennzeichen	
		df.applyPattern("+0.000;-0.000");  

		// komplexe Zahl (rot) malen
		g.setColor(Color.red);
		win.thickline(0.0,0.0,re,im);
		win.point(re,im);
		tfRe.setText(df.format(re));
		tfIm.setText(df.format(im));
		
		// Winkel (rot) zeichnen
		double angle = Math.atan2(im,re)/Math.PI*180.0;
		if (angle<0.0) angle = 360.0+angle+1.0;
		win.arc(0,0,0.5,angle);

		// Operation berechnen
		double re2=0.0,im2=0.0;
		double r=0,phi=0,re3,im3;
		switch (opState) {
		case I_KONJ:
			re2=re;
			im2=-im;
			break;
		case I_HOCH2:
			re2 = (re*re)-(im*im);
			im2 = 2*re*im;
			break;
		case I_HOCH3:
			double re22,im22;
			re22 = (re*re)-(im*im);
			im22 = 2*re*im;
			re2 = re22*re - im22*im;
			im2 = re22*im + im22*re;
			break;
		case I_HOCH1_2:
			r=Math.sqrt(re*re+im*im);
			phi=Math.atan2(im, re);
			re2 = Math.pow(r, 0.5)*Math.cos(phi/2);
			im2 = Math.pow(r, 0.5)*Math.sin(phi/2);
			break;
		case I_HOCH1_3:
			r=Math.sqrt(re*re+im*im);
			phi=Math.atan2(im, re);
			re2 = Math.pow(r, 1/3.0)*Math.cos(phi/3);
			im2 = Math.pow(r, 1/3.0)*Math.sin(phi/3);
			break;
		case I_HOCH1_4:
			r=Math.sqrt(re*re+im*im);
			phi=Math.atan2(im, re);
			re2 = Math.pow(r, 0.25)*Math.cos(phi/4);
			im2 = Math.pow(r, 0.25)*Math.sin(phi/4);
			break;
		}
		
		// komplexe Zahl (schwarz) malen
		g.setColor(Color.black);
		win.thickline(0.0,0.0,re2,im2);
		win.point(re2,im2);
		if (opState==OPERAND.I_HOCH1_2) {
			// 1 weitere Loesungen bei "hoch 1/2"
			win.thickline(0.0,0.0,-re2,-im2);
			win.point(-re2,-im2);
		}
		if (opState==OPERAND.I_HOCH1_3) {
			// 2 weitere Loesungen bei "hoch 1/3"
			re3 = Math.pow(r, 1/3.0)*Math.cos(phi/3+2*Math.PI/3);
			im3 = Math.pow(r, 1/3.0)*Math.sin(phi/3+2*Math.PI/3);
			win.thickline(0.0,0.0,re3,im3);
			win.point(re3,im3);
			re3 = Math.pow(r, 1/3.0)*Math.cos(phi/3+4*Math.PI/3);
			im3 = Math.pow(r, 1/3.0)*Math.sin(phi/3+4*Math.PI/3);
			win.thickline(0.0,0.0,re3,im3);
			win.point(re3,im3);
		}
		if (opState==OPERAND.I_HOCH1_4) {
			// 3 weitere Loesungen bei "hoch 1/4"
			win.thickline(0.0,0.0,-im2,re2);
			win.point(-im2,re2);
			win.thickline(0.0,0.0,-re2,-im2);
			win.point(-re2,-im2);
			win.thickline(0.0,0.0,im2,-re2);
			win.point(im2,-re2);		
		}
		
		// Winkel (schwarz) zeichnen
		double angle2 = Math.atan2(im2,re2)/Math.PI*180.0;
		if (angle2<0.0) angle2 = 360.0+angle2+1.0;
		win.arc(0,0,0.3,angle2);

        // diese Befehle sind streng verboten in paint():
		//		this.setVisible(true);
		//		this.setSize(600,400);
	}

	protected static class WindowClosingAdapter
	extends WindowAdapter
	{
		public WindowClosingAdapter()  {  }

		public void windowClosing(WindowEvent event)
		{
			event.getWindow().setVisible(false);
			event.getWindow().dispose();
			System.exit(0);
		}
	}
}